###################################################
# R code for "Big Data: New Tricks for Econometrics
# Journal of Economic Perspectives 28(2), 3-28            
# http://pubs.aeaweb.org/doi/pdfplus/10.1257/jep.28.2.3
# Hal R. Varian
###################################################

# data is from http://biostat.mc.vanderbilt.edu/wiki/Main/DataSets
# documentation in titanic3.txt
dat <- read.csv("titanic3.csv")

# for reproducibility
set.seed(1234)

# remove obs with missing data, and select a few predictors
titanic <- na.omit(dat[,c("survived","pclass","sex","age","sibsp")])

# create train and test sets (~40% for training, ~60% for testing)
titanic$randu <- runif(nrow(titanic),0,1)
titanic.train <- titanic[titanic$randu < .4,]
titanic.test <- titanic[titanic$randu >= .4,]

################################################
# ctree
################################################
library(party)
model.ctree <- ctree(as.factor(survived) ~ pclass + sex + age + sibsp, data = titanic.train)
plot(model.ctree)
# predict on test data
pred.ctree <- predict(model.ctree,newdata=titanic.test,type="response")
# confusion matrix
conf <- table(pred.ctree, titanic.test$survived)
conf
# proportions
prop.table(conf)
# error rate
(conf[1,2]+conf[2,1])/sum(conf)

##################################
# rpart
##################################
library(rpart)
# predict model
model.rpart <- rpart(as.factor(survived) ~ pclass + sex + age + sibsp, data = titanic.train)
# plot model
plot(model.rpart) # plot tree
text(model.rpart) # add labels to tree
# print complexity parameter table using cross validation
printcp(model.rpart)
# plot complexity parameter data
plotcp(model.rpart)
# choose complexity parameter to prune tree
model.pruned <- prune(model.rpart,cp=0.023)
# plot pruned tree
plot(model.pruned) # plot tree
text(model.pruned) # add lables

# compare models side by side
par(mfcol=c(1,2))
plot(model.rpart)
text(model.rpart)
plot(model.pruned)
text(model.pruned)
# predict using rpart model
pred.rpart <- predict(model.rpart,newdata=titanic.test,type="class")
conf <- table(pred.rpart, titanic.test$survived)
conf
prop.table(conf)
# error rate
(conf[1,2]+conf[2,1])/sum(conf)

# predict using pruned model
pred.pruned <- predict(model.pruned,newdata=titanic.test,type="class")
conf <- table(pred.pruned, titanic.test$survived)
conf
prop.table(conf)
# error rate
(conf[1,2]+conf[2,1])/sum(conf)

# compare to logit
model.logit <- glm(survived ~ pclass + sex + age + sibsp, data = titanic.train)
temp <- predict(model.logit,type="response",newdata=titanic.test)
pred.logit <- temp > .5
conf <- table(pred.logit, titanic.test$survived)
conf
prop.table(conf)
# error rate
(conf[1,2]+conf[2,1])/sum(conf)
